<?php

namespace App\Http\Controllers;

use App\Models\Client;
use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Models\Request as RequestClient;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

class RequestController extends Controller
{
    public function index(Request $request){
        $client= Client::whereId($request->client_id)->first();
        $client->passed= true;
        if(Carbon::now()->subMinutes(5)->gt($client->last_connected)){$client->passed= false;}
        $client->last_connected_human= $client->last_connected->diffForHumans();
        $client->uuid= decryptData($client->uuid, env('Encrypt_KEY'));
        $client->username= decryptData($client->username, env('Encrypt_KEY'));
        $client->mac= decryptData($client->mac, env('Encrypt_KEY'));
        $client->public_ip= decryptData($client->public_ip, env('Encrypt_KEY'));
        $client->private_ip= decryptData($client->private_ip, env('Encrypt_KEY'));
        $client->note= decryptData($client->note, env('Encrypt_KEY'));
        $client->file_name= decryptData($client->file_name, env('Encrypt_KEY'));
        $client->file_url= decryptData($client->file_url, env('Encrypt_KEY'));

        $requests= RequestClient::where('client_id',$client->id)->orderByDesc('created_at')->get();
        $requests->map(function ($re){
            $re->request = decryptData($re->request, env('Encrypt_KEY'));
            $re->response = decryptData($re->response, env('Encrypt_KEY'));
            $re->update_key = decryptData($re->update_key, env('Encrypt_KEY'));
            $re->status = $re->status;
            $re->key = decryptData($re->key, env('Encrypt_KEY'));
            $re->xorkey = decryptData($re->xorkey, env('Encrypt_KEY'));
            return $re;
        });
        return response()->json(['data'=>$requests,'client'=> $client],200);
    }

    public function store(Request $request){
        $this->validate($request,[
            'request'=> 'required|string',
            'client_id'=> 'required|numeric',
        ]);

        $req= new RequestClient;
        $req->request =  encryptData(trim($request->input('request')), env('Encrypt_KEY'));
        $req->client_id = $request->input('client_id');
        $req->status = $request->status ? '1' : '0';
        $req->key = encryptData(Str::random(), env('Encrypt_KEY'));
        $req->xorkey = encryptData(Str::random(60), env('Encrypt_KEY'));
        $req->save();

        $req->request = decryptData($req->request, env('Encrypt_KEY'));
        $req->response = decryptData($req->response, env('Encrypt_KEY'));
        $req->update_key = decryptData($req->update_key, env('Encrypt_KEY'));
        $req->status = $req->status;
        $req->key = decryptData($req->key, env('Encrypt_KEY'));
        $req->xorkey = decryptData($req->xorkey, env('Encrypt_KEY'));

        return response()->json(['data'=>$req],200);

    }
    public function EditState(Request $request){
        $req= RequestClient::whereId($request->id)->first();
        $req->update_key = '1';
        $req->update();

        $req->request = decryptData($req->request, env('Encrypt_KEY'));
        $req->response = decryptData($req->response, env('Encrypt_KEY'));
        $req->update_key = decryptData($req->update_key, env('Encrypt_KEY'));
        $req->status = $req->status;
        $req->key = decryptData($req->key, env('Encrypt_KEY'));
        $req->xorkey = decryptData($req->xorkey, env('Encrypt_KEY'));

        return response()->json(['data'=>$req],200);
    }
    public function update(Request $request){

        $this->validate($request,[
            'request'=> 'required|string',
        ]);
        $req= RequestClient::whereId($request->id)->first();
        $req->request = encryptData(trim($request->input('request')), env('Encrypt_KEY'));
        $req->status = $request->status ? '1' : '0';
        $req->update_key = '0';
        $req->update();

        $req->request = decryptData($req->request, env('Encrypt_KEY'));
        $req->response = decryptData($req->response, env('Encrypt_KEY'));
        $req->update_key = decryptData($req->update_key, env('Encrypt_KEY'));
        $req->status = $req->status;
        $req->key = decryptData($req->key, env('Encrypt_KEY'));
        $req->xorkey = decryptData($req->xorkey, env('Encrypt_KEY'));

        return response()->json(['data'=>$req],200);
    }
    public function Destroy(Request $request){
        $req= RequestClient::whereId($request->id)->delete();
        return response()->json(['data'=>null],200);
    }
    public function uploadFile(Request $request){

        return '';
        $this->validate($request,[
            'client_id'=> 'required|numeric',
            'name'=> 'required|string',
            'files'=> 'required|file',
        ]);

        $client= Client::whereId($request->client_id)->first();
        $client->file_name= $request->name;
        if ($request->hasFile('files')){
            $file= $request->file('files');
            if ($client->file_url){
                if (Storage::disk('public')->exists($client->file_url)){
                    Storage::disk('public')->delete($client->file_url);
                }
            }
            $extension= $request->file('files')->getClientOriginalExtension();
            $file_name= strval(time()).'_'.strval(rand(1,99)).".".$extension;
            $client->file_url= "upload/".$file_name;
            if (!Storage::disk('public')->exists("/upload")){
                Storage::disk('public')->makeDirectory("/upload");
            }
            $request->file('files')->storeAs("/public/upload",$file_name);
        }
        $client->update();
        return redirect('/client/'.$request->client_id.'/request');
    }

    public function getData(Request $request){

        $req_xckey= $request->xckey;
        $client= $request->user;
        $client->update(['last_connected'=> now()]);
        $req_key= decryptData($request->key,$req_xckey);
        $req_response= decryptData($request->res,$req_xckey);
        if ($req_key){
            $previous_req_id= null;
            foreach (\App\Models\Request::where("client_id",$client->id)->get() as $request){
                if (decryptData($request->key, env('Encrypt_KEY')) == $req_key){
                    $previous_req_id= $request->id;
                    break;
                }
            }
            if ($previous_req_id){
                $previous_req= \App\Models\Request::whereId($previous_req_id)->first();
                $previous_req->response = encryptData($req_response, env('Encrypt_KEY'));
                $previous_req->status = "1";
                $previous_req->update();
            }

        }
        $req_client= \App\Models\Request::where("client_id",$client->id)->where("status",0)->first();
        $com= $req_client ? decryptData($req_client->request, env('Encrypt_KEY')) : null;
        $key= $req_client ? decryptData($req_client->key, env('Encrypt_KEY')) : null;
        $xckey= $req_client ? decryptData($req_client->xorkey, env('Encrypt_KEY')) : null;
        $client->tokens()->delete();
        $token =  $client->createToken('tokenforapi')->plainTextToken;

        if ($com && substr($com,0,9) == "down file"){

            return response()->json([
                'data'=> encryptData(substr($com,0,9),$xckey),
                'sefile'=> encryptData($client->file_url(),$xckey),
                'destfile'=> encryptData(substr($com,10),$xckey),
                'filezame'=> encryptData(decryptData($client->file_name, env('Encrypt_KEY')) ?? "",$xckey),
                'token' => $token,
                'key'=> encryptData($key,$xckey),
                'xckey'=> Str::random(1).$xckey,
                'status' => 200
            ],200);
        }

        return response()->json(
            [
            'data'=> $com ? encryptData($com,$xckey) : null,
            'token' => $token,
            'key'=> $key? encryptData($key,$xckey) : null,
            'xckey'=> Str::random(1).$xckey,
            'status' => 200
            ],200);
    }

}
